

#Load the SOAP client support framework
package require SOAP

global MIDAS_SOAPAccessClient_Verbose
set MIDAS_SOAPAccessClient_Verbose 0


proc ::SOAP::MIDAS_SOAPAccessClient_Request {procVarName args} {
    upvar $procVarName procvar
    global MIDAS_SOAPAccessClient_Verbose

    set params  $procvar(params)
    set name    $procvar(name)
    set uri     $procvar(uri)

    if {$MIDAS_SOAPAccessClient_Verbose} {
         puts "::SOAP::MIDAS_SOAPAccessClient_Request called with $args"
         foreach member [array names procvar] {puts "procvar($member) =  $procvar($member)"}
    }
   
   set xml "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
   append xml "<SOAP-ENV:Envelope xmlns:xsi=\"http://www.w3.org/1999/XMLSchema-instance\" "
   append xml "xmlns:SOAP-ENV=\"$procvar(version)\" "
   append xml "xmlns:SOAP-ENC=\"$procvar(encoding)\" "
   append xml "xmlns:xsd=\"http://www.w3.org/1999/XMLSchema\" "
   append xml "SOAP-ENV:encodingStyle=\"$procvar(encoding)\">"

   append xml <SOAP-ENV:Body>
   append xml "<ns:$name xmlns:ns=\"$uri\">"

   set i 0; set j 0
   foreach arg $args {
       set ParamName [lindex $params $i]
       incr i
       set ParamType [lindex $params $i]
       incr i
       set ParamValue [lindex $args $j]
       incr j
       append xml "<$ParamName xsi:type=\"xsd:$ParamType\">$ParamValue</$ParamName>"
   }
   
   append xml "</ns:$name>"
   append xml </SOAP-ENV:Body>
   append xml </SOAP-ENV:Envelope>
      
   set xml [encoding convertto utf-8 $xml]          ;# make it UTF-8

    if {$MIDAS_SOAPAccessClient_Verbose} {puts "$xml"; puts ""}
      
   return $xml
}

proc ::SOAP::MIDAS_SOAPAccessClient_Reply {procVarName xml} {
    upvar $procVarName procvar
    global MIDAS_SOAPAccessClient_Verbose

    if {$MIDAS_SOAPAccessClient_Verbose} {
       puts "::SOAP::MIDAS_SOAPAccessClient_Reply called with $procVarName"
       puts "$xml"
       foreach member [array names procvar] {puts "procvar($member) =  $procvar($member)"}
        puts ""
    }

    if {$xml == {} && ![string match "http*" $procvar(proxy)]} {
        # This is probably not an error. SMTP and FTP won't return anything
        # HTTP should always return though (I think).
        return {}
    }

    set expected "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    if {[string compare -length 19 $xml $expected] == -1} {

#    unexpected reply or an error 

       return [::SOAP::parse_soap_response $procVarName $xml]
    }

#    This parser is VERY simple
#    Just look for the token <return and return whatever is the payload

#    Note - Using the Dom code to parse the XML  takes 10 ms

   global MIDAS_SOAPReturnCode MIDAS_SOAPFaultCode MIDAS_SOAPFaultString MIDAS_SOAPErrorCode MIDAS_SOAPErrorInfo

   set MIDAS_SOAPFaultCode   "NONE"
   set MIDAS_SOAPFaultString "NONE"
   set MIDAS_SOAPErrorCode   "NONE"
   set MIDAS_SOAPErrorInfo   "NONE"
   
   set MIDAS_SOAPReturnCode 0
   
   set result ""
   set i 0

    set frags [split $xml >]
    foreach frag $frags {
       if {[string compare -length 7 $frag <return] == 0 || [string compare -length 7 $frag <result] == 0} {
           set result "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
           break
       }
       if {[string compare -length 10 $frag <faultcode] == 0} {
           set MIDAS_SOAPFaultCode "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
           set MIDAS_SOAPReturnCode -1
       }
       if {[string compare -length 12 $frag <faultstring] == 0} {
           set MIDAS_SOAPFaultString "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
           set MIDAS_SOAPReturnCode -1
       }
       if {[string compare -length 10 $frag <errorcode] == 0} {
           set MIDAS_SOAPErrorCode "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
           set MIDAS_SOAPReturnCode -1
       }
       if {[string compare -length 10 $frag <errorinfo] == 0} {
           set MIDAS_SOAPErrorInfo "[lindex [split [lindex $frags [expr $i+1]] <] 0]"
           set MIDAS_SOAPReturnCode -1
       }
       incr i
    }
   
    if {$MIDAS_SOAPReturnCode == 0} then {
        return $result
    } else {
# insert-debug "MIDAS_SOAPErrorCode: $MIDAS_SOAPErrorCode"
# insert-debug "MIDAS_SOAPErrorInfo: $MIDAS_SOAPErrorInfo"
# insert-debug "MIDAS_SOAPFaultCode: $MIDAS_SOAPFaultCode"
# insert-debug "MIDAS_SOAPFaultString: $MIDAS_SOAPFaultString"
        error  "$MIDAS_SOAPErrorCode" "$MIDAS_SOAPErrorInfo" "$MIDAS_SOAPFaultCode-$MIDAS_SOAPFaultString"
    }
}

